//-----------------------------------------------------------------------------
//
//  Copyright (C) 2008-2011, Freescale Semiconductor, Inc. All Rights Reserved.
//  THIS SOURCE CODE, AND ITS USE AND DISTRIBUTION, IS SUBJECT TO THE TERMS
//  AND CONDITIONS OF THE APPLICABLE LICENSE AGREEMENT
//
//-----------------------------------------------------------------------------
//
//  File:  ecspi.cpp
//
//  Test eCSPI by SDK API.
//
//-----------------------------------------------------------------------------

#include <windows.h>
#include <ceddk.h>
#include "ecspibus.h"

#define dprintf NKDbgPrintfW

#define CSPI_DEVICE_NAME   _T("SPI1:")
// If there is the given SPIFLASH on board, the variable is 1 and notice CSPI_DEVICE_NAME
#define SPINORFLASH  0

#define NELEMS(x) (sizeof(x)/sizeof((x)[0]))

static HANDLE hCSPI;

void FlashWR();


CSPI_BUSCONFIG_T buscnfg =
{
    CSPI_CS2,   // ecspi channel, 0 1 2 3
    16000000,   // ecspi work freq
    64*32,      // burstlength
    FALSE,
    FALSE, 
    FALSE,
    0,
    FALSE        // usedma
};

HANDLE hEvent;
DWORD TxData[1024]; 
DWORD RxData[1024];

CSPI_XCH_PKT_T xchpkt =
{
    &buscnfg,
    TxData,
    RxData,
    64,         // data exchanging amount, 32bit unit; It must equal BurstLength/32 
                // or BurstLength/32 +1 (if BurstLength isnot multiple of 32bit)
    NULL,       // Event name
    0           // Event name length
};

//-----------------------------------------------------------------------------
//
// The main function
//-----------------------------------------------------------------------------
int WINAPI WinMain ( HINSTANCE hInstance, HINSTANCE hPrevInstance, 
    LPTSTR pCmdLine, int nCmdShow)
{
    DWORD i;
    DWORD error=0;

    UNREFERENCED_PARAMETER(hInstance);
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(pCmdLine);
    UNREFERENCED_PARAMETER(nCmdShow);

    dprintf(TEXT("\r\n*********   eCSPI Test    *********\r\n"));
    Sleep(3000);
    
    hEvent = CreateEvent(0, FALSE, FALSE, L"RX_EVENT");
    xchpkt.xchEvent = L"RX_EVENT";
    xchpkt.xchEventLength = sizeof(L"RX_EVENT");

   
    // Create eCSPI file handle
    hCSPI = CSPIOpenHandle(CSPI_DEVICE_NAME);

    if (hCSPI == INVALID_HANDLE_VALUE)
    {
        error = GetLastError();

        dprintf(TEXT("Create eCSPI file handle failed error=%d\r\n"), error);
        return -1;
    }

    dprintf(TEXT("Open eCSPI: %s\r\n"),CSPI_DEVICE_NAME );

    dprintf(TEXT("  Loopback Test\r\n"));
    CSPIEnableLoopback(hCSPI);

    for(i=0; i<(int)xchpkt.xchCnt; ++i)
    {
        TxData[i]= i;
        RxData[i]= 0;
    }

    if (!CSPIExchange(hCSPI, &xchpkt))
    {
        dprintf(TEXT("CSPIExchange failure.\r\n"));
        return 0;
    }

    WaitForSingleObject(hEvent, INFINITE);

     for(i=0; i<(int)xchpkt.xchCnt; ++i)
    {
        if(RxData[i]!=i)
        {
            dprintf(TEXT("error! received data[%d]= 0x%x, expected data=0x%x\r\n"),i,RxData[i],i);
            error +=1;
        }
    }

    CSPIDisableLoopback(hCSPI);

    if(0==error)    dprintf(TEXT("  Loopback Test Successfully!\r\n"));
    else    dprintf(TEXT("  Loopback Test Failed!\r\n"));


//*******************   SPIFLASH ACCESS  ***************************
#if SPINORFLASH
    FlashWR();
#endif

    // Close the CSPI file handle
    if (hCSPI != NULL)
    {
        CloseHandle(hCSPI);
    }

    dprintf(TEXT("\r\neCSPI Test quit.\r\n"));

    return 0;
}



//-----------------------------------------------------------------------------
//
// Operate eCSPI Norflash [M25P32]
//-----------------------------------------------------------------------------
void FlashWR()
{
    int i;

    dprintf(TEXT("\r\n              eCSPI Norflash Accessing\r\n"));
    xchpkt.pBusCnfg->ChannelSelect = CSPI_CS1;
    xchpkt.pBusCnfg->Freq =16000000;
    // Read chip  ID
    buscnfg.BurstLength= 32;
    xchpkt.xchCnt = 1;

    TxData[0] = 0x9F000000;         //9F is READ ID instrution
    for(i=0; i<(int)xchpkt.xchCnt-1; ++i)
    {
        TxData[i+1]= 0;
        RxData[i]= 0;
    }

    if (!CSPIExchange(hCSPI, &xchpkt))
    {
        dprintf(TEXT("CSPIExchange failure.\r\n"));
    }

    WaitForSingleObject(hEvent, INFINITE);

    dprintf(TEXT("SpiFlash ID= 0x%x\r\n"),RxData[0]&0xFFFFFF);


    // Read data from address 0x0
    xchpkt.pBusCnfg->BurstLength = 8*32;
    xchpkt.pBusCnfg->usedma= TRUE;
    xchpkt.xchCnt= 8;
    TxData[0] = 0x03000000;        //03 is read command, and the following 3 bytes are address
    for(i=0; i<(int)xchpkt.xchCnt-1; ++i)
    {
        TxData[i+1]= 0;
        RxData[i]= 0;
    }

    if (!CSPIExchange(hCSPI, &xchpkt))
    {
        dprintf(TEXT("CSPIExchange failure.\r\n"));
        return ;
    }

    WaitForSingleObject(hEvent, INFINITE);

     for(i=1; i<(int)xchpkt.xchCnt; ++i)
    {
        dprintf(TEXT("Spiflash data%d= 0x%x\r\n"),i,RxData[i]);
    }


    return ;

}

