// SerialWriteTest.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

// Size of the receive buffer
#define BUFFER_SIZE	32

// Character that stops the reception routine
#define STOP_CHAR '0'

// Standard output
#define TRACE(str, ...)	RETAILMSG(1, (TEXT(str), ##__VA_ARGS__))

static BOOL set_serial(HANDLE hCOM)
{
    DCB dcb;

    if (!GetCommState(hCOM, &dcb))
    {
        TRACE("ERROR: GetCommState returns false\r\n");
        return FALSE;
    }

    dcb.BaudRate    = CBR_115200;
    dcb.ByteSize    = 8;
    dcb.Parity      = NOPARITY;
    dcb.StopBits    = ONESTOPBIT;
    dcb.fParity     = FALSE;
    
//	dcb.fDtrControl = DTR_CONTROL_ENABLE;
//	dcb.fRtsControl = RTS_CONTROL_ENABLE;

    dcb.fDtrControl = DTR_CONTROL_DISABLE;
//    dcb.fRtsControl = RTS_CONTROL_DISABLE;
    dcb.fRtsControl = RTS_CONTROL_ENABLE;

    dcb.fOutxCtsFlow        = FALSE;
    dcb.fOutxDsrFlow        = FALSE;
    dcb.fDsrSensitivity     = FALSE;
    dcb.fOutX               = FALSE;
    dcb.fInX                = FALSE;
    dcb.fTXContinueOnXoff   = FALSE;

    if(!SetCommState(hCOM, &dcb))
    {
        TRACE("ERROR: SetCommState returns false\r\n");
        return FALSE;
    }
    else
        return TRUE;
}

//#define	COM_PORT	L"COM1:"
#define MAX_COMPORT	2
wchar_t* comport[MAX_COMPORT] = {L"N/A", L"COM1:"/*, L"COM2:", L"COM3:"*/};

int _tmain(int argc, TCHAR *argv[], TCHAR *envp[])
{
    BOOL    ret = TRUE;

	int		param = 1;
	
	if (argc == 2) {
		int temp = _wtoi(argv[1]);
		if (temp < MAX_COMPORT && temp > 0)
			param = temp;
	}

    // Allocate memory for the buffer
    char    buffer_out[BUFFER_SIZE] = {0};
    char    buffer_in[BUFFER_SIZE]  = {0};

    // Open the COM port
    HANDLE hCOM = CreateFile(comport[param], GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, 0,
        OPEN_EXISTING, 0, 0); 

    if (hCOM == INVALID_HANDLE_VALUE)
    {
        TRACE("ERROR: Failed to open COM port %d for reading & writing (error = %d)\r\n", param, GetLastError());
        ret = FALSE;
        goto cleanup;
    }
	TRACE("Opened COM port %d for reading & writing\r\n", param);

    // Set the port parameters
    if (!set_serial(hCOM))
    {
        TRACE("ERROR: Failed to set COM port parameters\r\n");
        ret = FALSE;
        goto cleanup;
    }

    Sleep(10000);

    // Initialize data buffer_out (a...z)
    for (int i = 0; i < BUFFER_SIZE - 1; i++)
    {
        buffer_out[i] = (BYTE)((int)('a') + i % ('z' - 'a'));
    }

    while(1)
    {
        // Write to the COM port
        DWORD bytes_written = 0;

        if (!WriteFile(hCOM, buffer_out, BUFFER_SIZE - 1, &bytes_written, NULL))
        {
            TRACE("ERROR: Failed to write to COM port (error = %d)\r\n", GetLastError());
            ret = FALSE;
            goto cleanup;
        }
        else
        {
            TRACE("INFO: OUT Buffer[%d] = [%S][%s]\r\n", bytes_written, buffer_out, buffer_out);
            //  Read from the COM port
            DWORD   bytes_read  = 0;

            if (!ReadFile(hCOM, buffer_in, BUFFER_SIZE - 1, &bytes_read, NULL))
            {
                TRACE("ERROR: Failed to read from COM port (error = %d)\r\n", GetLastError());
                ret = FALSE;
                goto cleanup;
            }
            else
            {
                TRACE("INFO: IN Buffer[%d] = [%S][%s]\r\n", bytes_read, buffer_in, buffer_in);
            }
			memset(buffer_in, 0, BUFFER_SIZE);
        }
        Sleep(10000);
    }

cleanup:
    if (hCOM != INVALID_HANDLE_VALUE)
        CloseHandle(hCOM);

    return 0;
}

